const ResizeSearchBox = {
  MIN_WIDTH: 40,          //minimum width of the searchbox
  THRESHOLD_HEIGHT: 100,  //max vertical distance travelable before we abort the drag gesture

  startup: function() {
    ResizeSearchBox.refreshBox();
  },

  refreshBox: function() {
    var sBox = document.getElementById("search-container");

    if(!sBox) {
      Components.classes["@mozilla.org/preferences-service;1"]
          .getService(Components.interfaces.nsIPrefBranch)
          .setIntPref("browser.search.searchbox.width", 280);  //reset mechanism
      return false;
    }

    if(/Macintosh/.test(navigator.userAgent) && /Firefox\/1\./.test(navigator.userAgent)) {
      if(!sBox.hasAttribute("macosx")) sBox.setAttribute("macosx", "true");
    }

    if(!sBox.width) {
      sBox.width = Components.classes["@mozilla.org/preferences-service;1"]
          .getService(Components.interfaces.nsIPrefBranch)
          .getIntPref("browser.search.searchbox.width");
      return false;
    }

    return true;
  },

  startDrag: function(event, ltr) {
    if(event.button != 0) return;

    var data = {
      startX: event.clientX,
      startY: event.clientY,
      ltr: ltr,
      origWidth: document.getElementById("search-container").width
    };

    data.whileDrag = function(e) {
      ResizeSearchBox.whileDrag(e, data);
    };

    data.endDrag = function(e) {
      ResizeSearchBox.endDrag(e, data);
    };

    window.addEventListener("mousemove", data.whileDrag, true);
    window.addEventListener("mouseup", data.endDrag, true);
  },

  whileDrag: function(event, data) {
    var dX = event.clientX - data.startX;
    var dY = event.clientY - data.startY;

     //If the pointer wanders too far while dragging
     //we will assume it was an accidental drag and abort.
    if(Math.abs(dY) > this.THRESHOLD_HEIGHT) {
      this.clearDrag(true, data);
      return;
    }

    if(!data.ltr) dX *= -1;

    if(data.origWidth - dX > this.MIN_WIDTH) {
      document.getElementById("search-container").width = data.origWidth - dX;
    }
  },

  endDrag: function(event, data) {
    this.clearDrag(false, data);
    Components.classes["@mozilla.org/preferences-service;1"]
        .getService(Components.interfaces.nsIPrefBranch)
        .setIntPref("browser.search.searchbox.width", document.getElementById("search-container").width);
  },

  clearDrag: function(revert, data) {
    window.removeEventListener("mousemove", data.whileDrag, true);
    window.removeEventListener("mouseup", data.endDrag, true);

    if(revert) document.getElementById("search-container").width = data.origWidth;
  }
}

window.addEventListener("load", ResizeSearchBox.startup, false);